@@ -11,7 +11,6 @@ import datetime as dt
 import nbxmpp
 from nbxmpp.modules.receipts import build_receipt
 from nbxmpp.namespaces import Namespace
-from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import StanzaHandler
@@ -20,6 +19,7 @@ from gajim.common import app
 from gajim.common import types
 from gajim.common.events import ReceiptReceived
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.contacts import BareContact
 from gajim.common.storage.archive import models as mod
 
 
@@ -35,7 +35,7 @@ class Receipts(BaseModule):
         ]
 
     def _process_message_receipt(self,
-                                 _con: types.xmppClient,
+                                 _con: types.NBXMPPClient,
                                  stanza: Message,
                                  properties: MessageProperties
                                  ) -> None:
@@ -52,7 +52,7 @@ class Receipts(BaseModule):
 
         if (properties.type.is_groupchat or
                 properties.is_self_message or
-                properties.is_carbon_message and properties.carbon.is_sent):
+                (properties.carbon is not None and properties.carbon.is_sent)):
 
             if properties.receipt.is_received:
                 # Don't propagate this event further
@@ -80,14 +80,16 @@ class Receipts(BaseModule):
                            properties.jid,
                            properties.receipt.id)
 
-            if properties.is_mam_message:
+            if properties.mam is not None:
                 timestamp = properties.mam.timestamp
             else:
                 timestamp = properties.timestamp
 
             timestamp = dt.datetime.fromtimestamp(
-                timestamp, dt.timezone.utc)
+                timestamp, dt.UTC)
 
+            assert properties.remote_jid is not None
+            assert properties.receipt.id is not None
             receipt_data = mod.Receipt(
                 account_=self._account,
                 remote_jid_=properties.remote_jid,
@@ -107,5 +109,7 @@ class Receipts(BaseModule):
         if properties.is_muc_pm:
             return True
 
-        contact = self._get_contact(JID.from_string(properties.jid.bare))
+        assert properties.jid is not None
+        contact = self._get_contact(properties.jid.new_as_bare())
+        assert isinstance(contact, BareContact)
         return contact.is_subscribed
